package mc600.gamemanager;

import java.text.*;
import java.util.Date;
import java.io.*;
import java.net.*;

public class Administrator {
    // tempo de espera entre campeonatos, milisegundos 
    private final static int PERIODICIDADE = 60000; 
    /**
     * Classe principal, responsavel pela organizacao do "campeonato"
     * Cria varias instancias do objeto Juiz, fazendo com que todos os
     * jogadores joguem entre si.
     *
     * Os jogadores devem estar conectados a uma porta TCP, dentro da faixa
     * determinada pelo objeto "Player", antes do inicio do campeonato.
     *
     * Ao final do campeonato gera um html com a classificacao dos jogadores.
     *
     */
    public Administrator() {
	super();
    }

    /**
     * Starts the application.
     * @param args an array of command-line arguments
     */
    public static void main (java.lang.String[] args) 
	throws IOException {
	
	while(true){

	    System.err.println("Acordando...");
	
	    // aloca todos os jogadores disponiveis
	    // dentro da faixa de ips determinada.
	    PlayerQueue players = 
		summonPlayers(PlayerClient.MINIP,PlayerClient.MAXIP);

	    if (players.nPlayers() > 1) {
		// inicia campeonato, promovendo jogos entre
		// todos os participantes
		executeGames(players);

		// Ordena vetor de jogadores
		// em ordem decrescente de classificacao
		players.sort();
 	
		// Grava html com classificacao
		gravaHtml(players,"stick.html");
	    } else
		System.err.println("Campeonato nao realizado");

	    try{
		Thread.sleep(PERIODICIDADE);
	    }
	    catch (InterruptedException e){
		System.err.println("Thread interrompida.");
	    }
	
	}
	
	
    }

    /**
     * Aloca em vetor jogadores encontrados entre minIp e maxIp. 
     * Seta numero de jogadores.
     *
     *
     * Creation date: (3/22/00 10:56:13 PM)
     * @return mc600.gamemanager.PlayerServer[]
     * @param minIp int
     * @param maxIp int
     */
    private static PlayerQueue summonPlayers(int minIP, int maxIP) {

	//inicializa vetor de jogadores de modo que o numero seja sempre par
	int vectorSize = maxIP - minIP;
	vectorSize += (vectorSize%2!=0?1:0);
	PlayerClient players[] = new PlayerClient[vectorSize];

	System.err.println("Verificando intervalo de portas: " + 
			   minIP + " a " + maxIP);
	
	// numero de jogadores validos encontrados
	int nPlayers=0;

	for(int i=minIP;i<=maxIP;i++){

	    try{

		// aloca jogador
		players[nPlayers] = 
		    new PlayerClient(InetAddress.getByName ("localhost"), i);

		// derruba socket liberando para o proximo jogo
		players[nPlayers].shutdown();

		System.err.println(i+":"+players[nPlayers].getPlayerName());
		players[nPlayers].openLog();

		nPlayers++;
			
	    } catch(Exception e) {
		// so' testando portas
	    }		
	}
	
	System.err.println("Encontrado(s) " + 
			   nPlayers + " jogador(es) valido(s).");
	
	return new PlayerQueue(players, nPlayers);
    }
    
    /**
     * Funcao inicia campeonato
     * Creation date: (3/24/00 2:54:57 AM)
     */
    private static void executeGames (PlayerQueue players) 
	throws IOException {

	System.err.println("Iniciando Campeonato");
	
        // faz todas as rodadas
	for (int i=0; i < players.nPlayers(); i++) {
	    for (int j=0; j < players.nPlayers(); j++){
		if (i==j)
		    continue;
		System.err.print(".");
		new Judge(players.getPlayer(i),players.getPlayer(j));
	    }
	    System.err.println();
	}
	System.err.println("Campeonato terminado");
    }

    /**
     * Metodo grava html com ranking dos participantes.
     *
     * Creation date: (3/26/00 8:58:55 PM)
     * @param players mc600.gamemanager.PlayerQueue[]
     */
    private static void gravaHtml(PlayerQueue players, String arq_out) {

 	SimpleDateFormat formatter = 
	    new SimpleDateFormat ("dd/MM/yyyy HH:mm:ss");

	Date currentTime = new Date();
	
	String dateString = formatter.format(currentTime);
	String nextChamp  = formatter.format
	    (new Date(System.currentTimeMillis() + PERIODICIDADE));

	// santo codigo porco, batman!!
	String html_top =
	    new String("<html>\n" +
		       "<head>\n" +
		       "<title>Campeonato Java</title>\n" +
		       "<meta http-equiv=\"Content-Type\"" +
		       "content=\"text/html; charset=iso-8859-1\">\n" +
		       "</head>\n\n" +
		       "<body bgcolor=\"#FFFFFF\">\n" + 
		       "<p><font size=\"5\" " +
		       "face=\"Arial, Helvetica, sans-serif\">" +
		       "MC600 Apresenta: Connect Four <br>\n"+ 
		       " </font></p>\n" +
		       "<hr>\n" + 
		       "<p>Data e hora do campeonato mais recente: " +
		       dateString + "<br>\n" + 
		       "   Data e hora do pr&oacute;ximo campeonato: " +
		       nextChamp + "</p>\n");
	
	String html_tableheader = 
	    new String("<table width=\"75%\" border=\"1\">\n" +
		       "<tr>\n" +
		       "<td width=\"10%\">Boas Partidas</td>\n" +
		       "<td width=\"10%\">Nmero de Vitrias</td>\n" +
		       "<td width=\"10%\">Nmero de Empates</td>\n" +
		       "<td width=\"10%\">Jogada Invlida</td>\n" +
		       "<td width=\"10%\">Tempo Esgotado</td>\n" +
		       "<td width=\"10%\">Outros Erros</td>\n" +
		       "<td width=\"10%\">Total de Pontos</td>\n" +
		       "<td width=\"10%\">Nota</td>\n" +
		       "<td width=\"40%\">Nome do jogador</td>\n" +
		       "</tr>\n");
	
	
	String html_bottom = 
	    new String("</table>\n" +
		       "<p>Nome do jogador  link para seu log</p>\n"  +
		       "<p><a href=\"mc600.zip\">"+
		       "Cdigo fonte do sistema</a></p>\n" +
		       "</body>\n" +
		       "</html>\n");

	// calcula subsidios para a nota
	// e move .tmp para .log

	PlayerClient p;
	int n = players.nPlayers();
	int maior = 0;
	int menor = 4*(n-1);
	
	for (int i=0; i < n; i++){
	    
	    p = players.getPlayer(i);
	    int boasPartidas = p.getBoasPartidas();
	    int points = 2*p.getVictoryCount() + p.getDrawCount();
	    // se boasPartidas > 90% do total ...
	    if ( 10*boasPartidas >= 9*2*(n-1) ) {
		menor = Math.min(menor,points);
		maior = Math.max(maior,points);
	    }
	    
	    // salva log
	    p.closeLog();
	    File old = new File(p.getPlayerName() + ".tmp");
	    File log = new File(p.getPlayerName() + ".log");
	    if (!old.renameTo(log)) 
		System.err.println("Could not rename " +
				   p.getPlayerName() +
				   "'s log file");
	}
	
	System.err.println("Ad: found " + n + " players");
	
	// grava arquivo html
	
	try {
	    
	    // Abre o arquivo arq_out
	    
	    System.err.println("Criando arquivo com classificacao: " 
			       + arq_out);
	    
	    DataOutputStream arq = 
		new DataOutputStream( new FileOutputStream (arq_out) );
	    
	    //Grava parte fixa
	    arq.writeBytes(html_top);
	    
	    // Grava numero de jogadores, min. e max. points
	    arq.writeBytes("<p>Total: " + n + " jogadores\n" +
			   "<br>Min. pontos: " + menor +
			   "<br>Max. pontos: " + maior + "</p>");
	    
	    // grava tabela e lista de usuarios
	    arq.writeBytes(html_tableheader);
	    
	    for(int i=n-1; i>=0; i--) {
		// calcula nota
		p = players.getPlayer(i);
		long notaVezes10;
		int boasPartidas = p.getBoasPartidas();
		int victories    = p.getVictoryCount();
		int draws        = p.getDrawCount();
		int points       = 2*victories + draws;
		if ( 10*boasPartidas >= 9*2*(n-1) )
		    notaVezes10 = 
			Math.round(50.0 + 50.0 *
				   (points-menor)/(maior-menor));
		else
		    notaVezes10 = 
			Math.round(50.0*boasPartidas/(2*(n-1)));
		
		// imprime
		String name = p.getPlayerName();
		arq.writeBytes("<tr>\n" + 
			       "<td width=\"10%\">" + 
			       boasPartidas + 
			       "</td>\n" + 
			       "<td width=\"10%\">" + 
			       victories +
			       "</td>\n" + 
			       "<td width=\"10%\">" + 
			       draws + 
			       "</td>\n" + 
			       "<td width=\"10%\">" + 
			       p.getJogInvalida() + 
			       "</td>\n" + 
			       "<td width=\"10%\">" + 
			       p.getTempEsgotado() + 
			       "</td>\n" + 
			       "<td width=\"10%\">" + 
			       p.getOutrosErros() + 
			       "</td>\n" + 
			       "<td width=\"10%\">" + 
			       points + 
			       "</td>\n" + 
			       "<td width=\"10%\">" + 
			       ((float)notaVezes10)/10 + 
			       "</td>\n" + 
			       "<td width=\"40%\">" + 
			       "<a href=\"" + name + ".log\">" +
			       name + "</a>" +
			       "</td>\n" + 
			       "</tr>\n");
	    }
	    
	    // grava parte final do arquivo
	    arq.writeBytes(html_bottom);	

	    arq.close();
	} catch (IOException e) {
	    System.err.println("Erro no arquivo " + arq_out
			       + e.getMessage());
	}
    }
}
