package mc600.gamemanager;

import java.io.*;
import java.net.*;

/**
 * O objeto PlayerServer representa o seu jogador.  Voce deve ter uma
 * instancia desta classe para poder jogar.
 *
 * O procedimento e' simples. Para programar
 * o seu servidor voce deve seguir os seguintes passos:
 *
 *
 * 1 - Criar uma instancia do objeto PlayerServer,
 *     passando como parametro o seu identificador e,
 *     opcionalmente, o "host" do gerenciador.
 * 
 * 2 - No corpo do seu programa voce deve incluir
 *     uma chamada para "waitClient()", que colocara
 *     o seu programa numa porta IP, escolhida automaticamente,
 *     esperando pela chamada do Gerenciador.
 * 
 * 3 - Ao chegar o Gerenciador, a funcao waitClient recebera'
 *     o primeiro lance da partida, ou a string Player.STARTCMD,
 *     significando que voce e' o primeiro a jogar.
 *
 * 4 - Voce tem um tempo determinado por Player.MAXRESPONSETIME
 *     para decidir qual o seu lance, e entao
 *     repassa-lo para o servidor.  Para isto voce deve utilizar
 *     o comando  play(lance) , passando o seu
 *     lance como argumento. Esta funcao enviara' o seu lance
 *     para o gerenciador, retornando a resposta do oponente.
 * 
 *
 * Caso voce submeta um lance invalido, ou demore para responder,
 * a conexao sera derrubada, e voce perde automaticamente a partida
 * (nao o campeonato).
 *
 *
 * Ao final do jogo a conexao sera derrubada.  Para o proximo jogo
 * sera' iniciada outra conexao.
 *
 * ATENCAO: Voce deve fazer um "loop" principal, que sempre chame
 * a funcao "waitClient" apos o fechamento da conexao atual,
 * caso contrario o seu jogador so' jogara' a primeira partida!!
 *
 *
 * EXEMPLO: O Objeto "ServerTest" ilustra o corpo de um programa
 * que faz essas coisas...
 *
 *
 * Creation date: (3/8/00 11:09:14 PM)
 * @author: Celso Cardoso, EC98, UNICAMP
 */

public class PlayerServer extends Player {
    private java.net.ServerSocket sktServer;
    private Socket skt;

    /**
     * Insert the method's description here.
     * Creation date: (3/25/00 3:49:58 AM)
     * @param id java.lang.String
     */
    public PlayerServer(String host, String id) throws Exception{
	
	super(id);

	int port;

	// Encontra primeira porta livre... Q&D

	for (port=MINIP; port<=MAXIP; port++){

	    try {

		// Cria o socket servidor na porta especificada e
		// no host especificado (segundo argumento e' numero
		// de conexoes pendentes admitidas = 1)
		sktServer =
		    new ServerSocket (port,1,
				      InetAddress.getByName(host));
		break;

	    } catch(Exception e) {
		// ignora excecoes...
		// esta apenas procurando porta livre..
	    } 

	}		

	if (port > MAXIP)
	    // nao encontrou porta livre!!
	    throw new Exception("Porta livre nao encontrada em " + host
				+ " (" + MINIP + "-" + MAXIP + ")");
	
    }

    /**
     * Insert the method's description here.
     * Creation date: (3/25/00 3:49:58 AM)
     * @param id java.lang.String
     */
    public PlayerServer(String id) throws Exception{
	
	super(id);

	int port;

	// Encontra primeira porta livre... Q&D

	for (port=MINIP; port<=MAXIP; port++){

	    try {

		// Cria o socket servidor na porta especificada
		sktServer = new ServerSocket (port);
		break;

	    } catch(Exception e) {
		// ignora excecoes...
		// esta apenas procurando porta livre..
	    } 

	}		

	if (port > MAXIP)
	    // nao encontrou porta livre!!
	    throw new Exception("Nenhuma porta livre encontrada!");
	
    }

    /**
     * Funcao utilizada para enviar e receber os lances do servidor
     *
     * Creation date: (3/23/00 1:03:54 AM)
     * @return java.lang.String
     * @param move java.lang.String
     * @exception java.lang.Exception The exception description.
     */
    public String play(String move) throws Exception {

	byte[] bmove = new byte[MOVESIZE];

	
	if (move.length() != MOVESIZE)

	    throw new Exception("Tamanho de lance invalido.");

	if (skt == null)
		
	    throw new Exception("Socket nao inicializado.");

	// Cria os buffers de saida e entrada
	DataInputStream dataIn =
	    new DataInputStream (skt.getInputStream ());
   	DataOutputStream dataOut =
	    new DataOutputStream (skt.getOutputStream ());

   	// Envia o lance
   	dataOut.writeBytes(move);
   	dataOut.flush();
 		  	   	
	// Le o proximo comando
	dataIn.readFully(bmove);
		
	return (new String(bmove));		
    }

    public void shutdown() throws IOException {

	skt.close();

    }

    /**
     * Esta funcao aguarda cliente, retornando string contendo:
     *
     *  Player.STARTCMD : caso o primeiro lance seja deste jogador
     *  "XY" : coordenada do primeiro lance,
     *         caso este jogador seja o segundo.
     *
     *
     * Creation date: (3/23/00 1:05:20 AM)
     * @return java.lang.String
     * @exception java.lang.Exception The exception description.
     */
    public String waitClient() throws Exception {

	byte[] move = new byte[MOVESIZE];

	// Cria socket para intercambio dos dados
	skt = sktServer.accept (); 

	skt.setTcpNoDelay(true);
		
	// Cria os buffers de saida e entrada
	DataInputStream dataIn = 
	    new DataInputStream (skt.getInputStream ());
	DataOutputStream dataOut = 
	    new DataOutputStream (skt.getOutputStream ());
   	   
	// Envia o Magic.. e' uma string especial so'
	// para confirmar que quem esta' do outro lado
	// conhece o protocolo ...
	dataOut.writeBytes(MAGIC);

	// Envia o tamanho do playername
	dataOut.writeInt(getPlayerName().length());

	// Envia o nome do jogador
	dataOut.writeBytes(getPlayerName());
	dataOut.flush();
	   	   
	// Le o proximo comando
	dataIn.readFully(move,0,MOVESIZE);

	return (new String(move));
    }
}
